@@ -63,6 +63,10 @@ Example Authorization server::
     # Add Grants you want to support
     auth_controller.add_grant(oauth2.grant.AuthorizationCodeGrant())
     auth_controller.add_grant(oauth2.grant.ImplicitGrant())
+    
+    # Add refresh token capability and set expiration time of access tokens
+    # to 30 days
+    auth_controller.add_grant(oauth2.grant.RefreshToken(expires_in=2592000))
 
     # Wrap the controller with the Wsgi adapter
     app = oauth2.web.Wsgi(server=auth_controller)
@@ -88,17 +92,21 @@ from oauth2.error import OAuthInvalidError, OAuthUserError
 from oauth2.web import Request, Response
 from oauth2.tokengenerator import Uuid4
 
-VERSION = "0.3.2"
+VERSION = "0.4.0"
 
 class AccessToken(object):
     """
     An access token and associated data.
     """
-    def __init__(self, client_id, token, data={}, scopes=[]):
-        self.client_id = client_id
-        self.token     = token
-        self.data      = data
-        self.scopes    = scopes
+    def __init__(self, client_id, grant_type, token, data={}, expires_at=None,
+                 refresh_token=None, scopes=[]):
+        self.client_id     = client_id
+        self.grant_type    = grant_type
+        self.token         = token
+        self.data          = data
+        self.expires_at    = expires_at
+        self.refresh_token = refresh_token
+        self.scopes        = scopes
 
 class AuthorizationCode(object):
     """
@@ -134,26 +142,27 @@ class Client(object):
         return uri in self.redirect_uris
 
 class AuthorizationController(object):
-    
     authorize_path = "/authorize"
     token_path = "/token"
-    """
-    Endpoint of requests to the OAuth 2.0 server.
     
-    :param access_token_store: An object that implements methods defiend by
-                               :class:`oauth2.store.AccessTokenStore`.
-    :param auth_code_store: An object that implements methods defiend by
-                            :class:`oauth2.store.AuthTokenStore`.
-    :param client_store: An object that implements methods defiend by
-                         :class:`oauth2.store.ClientStore`.
-    :param site_adapter: An object that implements methods defiend by
-                         :class:`oauth2.web.SiteAdapter`.
-    :param token_generator: Object to generate unique tokens.
-    :param response_class: Class of the response object.
-                           Default: :class:`oauth2.web.Response`.
-    """
     def __init__(self, access_token_store, auth_code_store, client_store,
                  site_adapter, token_generator, response_class=Response):
+        """
+        Endpoint of requests to the OAuth 2.0 server.
+        
+        :param access_token_store: An object that implements methods defiend by
+                                   :class:`oauth2.store.AccessTokenStore`.
+        :param auth_code_store: An object that implements methods defiend by
+                                :class:`oauth2.store.AuthTokenStore`.
+        :param client_store: An object that implements methods defiend by
+                             :class:`oauth2.store.ClientStore`.
+        :param site_adapter: An object that implements methods defiend by
+                             :class:`oauth2.web.SiteAdapter`.
+        :param token_generator: Object to generate unique tokens.
+        :param response_class: Class of the response object.
+                               Default: :class:`oauth2.web.Response`.
+    
+        """
         self.grant_types    = []
         self._input_handler = None
         
@@ -168,6 +177,9 @@ class AuthorizationController(object):
         """
         Adds a Grant that the server should support.
         """
+        if hasattr(grant, "expires_in"):
+            self.token_generator.expires_in = grant.expires_in
+        
         self.grant_types.append(grant)
     
     def dispatch(self, request, environ):
