@@ -28,12 +28,24 @@ So there are two remaining parties:
 
 """
 from oauth2.error import OAuthInvalidError, OAuthUserError, OAuthClientError,\
-    ClientNotFoundError, UserNotAuthenticated
+    ClientNotFoundError, UserNotAuthenticated, AccessTokenNotFound
 from oauth2.compatibility import urlencode, quote
 import json
 import time
 from oauth2 import AuthorizationCode, AccessToken
 
+def json_error_response(error, response):
+    """
+    Formats an error as a response containing a JSON body.
+    """
+    msg = {"error": error.error, "description": error.explanation}
+
+    response.status_code = 400
+    response.add_header("Content-Type", "application/json")
+    response.body = json.dumps(msg)
+    
+    return response
+
 class Scope(object):
     """
     Handling of the "scope" parameter in a request.
@@ -56,6 +68,20 @@ class Scope(object):
         
         self.default = default
     
+    def compare(self, previous_scopes):
+        """
+        Compares the scopes read from request with previously issued scopes.
+        
+        :param previous_scopes: A list of scopes.
+        :return: ``True``
+        """
+        for scope in self.scopes:
+            if scope not in previous_scopes:
+                raise OAuthInvalidError(error="invalid_scope",
+                                        explanation="Invalid scope parameter in request")
+        
+        return True
+    
     def parse(self, request):
         """
         Parses scope value in given request.
@@ -289,13 +315,14 @@ class AuthorizationCodeTokenHandler(GrantHandler):
         self.client_id     = None
         self.client_secret = None
         self.code          = None
+        self.data          = {}
         self.redirect_uri  = None
         self.scopes        = []
         
         self.access_token_store = access_token_store
-        self.auth_code_store   = auth_token_store
-        self.client_store       = client_store
-        self.token_generator    = token_generator
+        self.auth_code_store = auth_token_store
+        self.client_store = client_store
+        self.token_generator = token_generator
     
     def read_validate_params(self, request):
         """
@@ -324,16 +351,21 @@ class AuthorizationCodeTokenHandler(GrantHandler):
         
         Calls `oauth2.store.AccessTokenStore` to persist the token.
         """
-        token = self.token_generator.generate()
+        token_data = self.token_generator.create_access_token_data()
         
-        result = {"access_token": token, "token_type": "Bearer"}
-        
-        access_token = AccessToken(client_id=self.client_id, token=token,
+        access_token = AccessToken(client_id=self.client_id, data=self.data,
+                                   grant_type=AuthorizationCodeGrant.grant_type,
+                                   token=token_data["access_token"],
                                    scopes=self.scopes)
         
+        if "refresh_token" in token_data:
+            expires_at = int(time.time()) + token_data["expires_in"]
+            access_token.expires_at = expires_at
+            access_token.refresh_token = token_data["refresh_token"]
+        
         self.access_token_store.save_token(access_token)
         
-        response.body = json.dumps(result)
+        response.body = json.dumps(token_data)
         response.status_code = 200
         
         response.add_header("Content-type", "application/json")
@@ -341,17 +373,7 @@ class AuthorizationCodeTokenHandler(GrantHandler):
         return response
     
     def redirect_oauth_error(self, error, response):
-        """
-        Returns a response with status "400" and the error as JSON in case an
-        error during the oauth process occurred.
-        """
-        msg = {"error": error.error}
-        
-        response.status_code = 400
-        response.add_header("Content-type", "application/json")
-        response.body = json.dumps(msg)
-        
-        return response
+        return json_error_response(error, response)
     
     def _read_params(self, request):
         self.client_id     = request.post_param("client_id")
@@ -403,6 +425,7 @@ class AuthorizationCodeTokenHandler(GrantHandler):
             raise OAuthInvalidError(error="invalid_grant",
                                   explanation="Authorization code has expired")
         
+        self.data = stored_code.data
         self.scopes = stored_code.scopes
 
 class AuthorizationCodeGrant(GrantHandlerFactory, ScopeGrant):
@@ -418,6 +441,9 @@ class AuthorizationCodeGrant(GrantHandlerFactory, ScopeGrant):
         
         auth_controller.add_grant_type(AuthorizationCodeGrant())
     """
+    
+    grant_type = "authorization_code"
+    
     def __call__(self, request, server):
         if (request.post_param("grant_type") == "authorization_code"
             and request.path == server.token_path):
@@ -439,6 +465,7 @@ class AuthorizationCodeGrant(GrantHandlerFactory, ScopeGrant):
         return None
 
 class ImplicitGrant(GrantHandlerFactory, ScopeGrant):
+    
     """
     Implementation of the Implicit Grant auth flow.
     
@@ -451,6 +478,9 @@ class ImplicitGrant(GrantHandlerFactory, ScopeGrant):
         
         auth_controller.add_grant_type(ImplicitGrant())
     """
+    
+    grant_type = "implicit"
+    
     def __call__(self, request, server):
         response_type = request.get_param("response_type")
         
@@ -487,8 +517,9 @@ class ImplicitGrantHandler(AuthRequestMixin, GrantHandler):
         
         token = self.token_generator.generate()
         
-        access_token = AccessToken(client_id=self.client_id, token=token,
-                                   data=user_data,
+        access_token = AccessToken(client_id=self.client_id,
+                                   grant_type=ImplicitGrant.grant_type,
+                                   token=token, data=user_data,
                                    scopes=self.scope_handler.scopes)
         
         self.access_token_store.save_token(access_token)
@@ -534,6 +565,9 @@ class ResourceOwnerGrant(GrantHandlerFactory, ScopeGrant):
         
         auth_controller.add_grant_type(ResourceOwnerGrant())
     """
+    
+    grant_type = "password"
+    
     def __call__(self, request, server):
         """
         Checks if the incoming request can be handled by the
@@ -576,22 +610,27 @@ class ResourceOwnerGrantHandler(GrantHandler):
         user_data = self.site_adapter.authenticate(request, environ,
                                                    self.scope_handler.scopes)
         
-        token = self.token_generator.generate()
+        token_data = self.token_generator.create_access_token_data()
         
-        access_token = AccessToken(client_id=self.client_id, token=token,
+        access_token = AccessToken(client_id=self.client_id,
+                                   token=token_data["access_token"],
+                                   grant_type=ResourceOwnerGrant.grant_type,
                                    data=user_data,
                                    scopes=self.scope_handler.scopes)
         
+        if "refresh_token" in token_data:
+            expires_at = int(time.time()) + token_data["expires_in"]
+            access_token.expires_at = expires_at
+            access_token.refresh_token = token_data["refresh_token"]
+        
         self.access_token_store.save_token(access_token)
         
-        response_body = {"access_token": token, "token_type": "bearer"}
-        
         if self.scope_handler.send_back is True:
-            response_body["scope"] = " ".join(self.scope_handler.scopes)
+            token_data["scope"] = " ".join(self.scope_handler.scopes)
         
         response.add_header("Content-Type", "application/json")
         response.status_code = 200
-        response.body = json.dumps(response_body)
+        response.body = json.dumps(token_data)
         
         return response
     
@@ -623,13 +662,152 @@ class ResourceOwnerGrantHandler(GrantHandler):
         return True
     
     def redirect_oauth_error(self, error, response):
-        """
-        Formats an error as a response containing a JSON body.
-        """
-        msg = {"error": error.error, "description": error.explanation}
+        return json_error_response(error, response)
+
+class RefreshToken(GrantHandlerFactory, ScopeGrant):
+    """
+    Handles requests for refresk tokens as defined in
+    http://tools.ietf.org/html/rfc6749#section-6.
+    
+    Adding a Refresh Token to the :class:`oauth2.AuthorizationController` like
+    this::
+    
+        auth_controller = AuthorizationController()
         
-        response.status_code = 400
-        response.add_header("Content-Type", "application/json")
-        response.body = json.dumps(msg)
+        auth_controller.add_grant_type(RefreshToken(expires_in=600))
+        
+    will cause :class:`oauth2.grant.AuthorizationCodeGrant` and
+    :class:`oauth2.grant.ResourceOwnerGrant` to include a refresh token and
+    expiration in the response.
+    """
+    
+    grant_type = "refresh_token"
+    
+    def __init__(self, expires_in, default_scope=None, scopes=None,
+                 scope_class=Scope):
+        self.expires_in = expires_in
+        
+        ScopeGrant.__init__(self, default_scope=default_scope, scopes=scopes,
+                            scope_class=scope_class)
+    def __call__(self, request, server):
+        """
+        Determines if the current request requests a refresh token.
+        
+        :return: An instance of :class:`RefreshTokenHandler`.
+        """
+        if request.path != server.token_path:
+            return None
+        
+        if request.post_param("grant_type") != "refresh_token":
+            return None
+        
+        return RefreshTokenHandler(access_token_store=server.access_token_store,
+                                   client_store=server.client_store,
+                                   scope_handler=self._create_scope_handler(),
+                                   token_generator=server.token_generator)
+
+class RefreshTokenHandler(GrantHandler):
+    """
+    Validates an incoming request and issues a new access token.
+    """
+    def __init__(self, access_token_store, client_store, scope_handler,
+                 token_generator):
+        self.access_token_store = access_token_store
+        self.client_store       = client_store
+        self.scope_handler      = scope_handler
+        self.token_generator    = token_generator
+        
+        self.client_id     = None
+        self.data          = {}
+        self.refresh_token = None
+    
+    def process(self, request, response, environ):
+        """
+        Create a new access token.
+        
+        :param request: The incoming :class:`oauth2.web.Request`.
+        :param response: The :class:`oauth2.web.Response` that will be returned
+                         to the client.
+        :param environ: A ``dict`` containing data of the environment.
+        
+        :return: :class:`oauth2.web.Response`
+        
+        """
+        expires_in = self.token_generator.expires_in
+        expires_at = int(time.time()) + expires_in
+        token = self.token_generator.generate()
+        
+        access_token = AccessToken(client_id=self.client_id, token=token,
+                                   grant_type=RefreshToken.grant_type,
+                                   data=self.data, expires_at=expires_at,
+                                   scopes=self.scope_handler.scopes)
+        self.access_token_store.save_token(access_token)
+        
+        response_data = {"access_token": token, "expires_in": expires_in,
+                         "token_type": "Bearer"}
+        
+        response.add_header("Content-type", "application/json")
+        response.body = json.dumps(response_data)
         
         return response
+    
+    def read_validate_params(self, request):
+        """
+        Validate the incoming request.
+        
+        :param request: The incoming :class:`oauth2.web.Request`.
+        
+        :return: Returns ``True`` if data is valid.
+        
+        :raises: :class:`oauth2.error.OAuthInvalidError`
+        
+        """
+        self.client_id = request.post_param("client_id")
+        
+        if self.client_id is None:
+            raise OAuthInvalidError(error="invalid_request",
+                                    explanation="Missing client_id in request body")
+        
+        client_secret = request.post_param("client_secret")
+        
+        if client_secret is None:
+            raise OAuthInvalidError(error="invalid_request",
+                                    explanation="Missing client_secret in request body")
+        
+        self.refresh_token = request.post_param("refresh_token")
+        
+        if self.refresh_token is None:
+            raise OAuthInvalidError(error="invalid_request",
+                                    explanation="Missing refresh_token in request body")
+        
+        try:
+            client = self.client_store.fetch_by_client_id(self.client_id)
+            
+            if client.secret != client_secret:
+                raise OAuthInvalidError(error="invalid_request",
+                                        explanation="Invalid client secret")
+        except ClientNotFoundError:
+            raise OAuthInvalidError(error="invalid_request",
+                                    explanation="Unknown client")
+        
+        try:
+            access_token = self.access_token_store.fetch_by_refresh_token(
+                self.refresh_token
+            )
+        except AccessTokenNotFound:
+            raise OAuthInvalidError(error="invalid_request",
+                                    explanation="Invalid refresh token")
+        
+        if access_token.expires_at < int(time.time()):
+            raise OAuthInvalidError(error="invalid_request",
+                                    explanation="Invalid refresh token")
+        
+        self.data = access_token.data
+        
+        self.scope_handler.parse(request)
+        self.scope_handler.compare(access_token.scopes)
+        
+        return True
+    
+    def redirect_oauth_error(self, error, response):
+        return json_error_response(error, response)
