@@ -1,23 +1,20 @@
-from mock import Mock, call
+from mock import Mock, call, patch
 import json
 from oauth2.test import unittest
 from oauth2.web import Request, Response, SiteAdapter
 from oauth2.grant import ImplicitGrantHandler, AuthorizationCodeAuthHandler,\
     AuthRequestMixin, AuthorizationCodeTokenHandler, ImplicitGrant,\
     AuthorizationCodeGrant, ResourceOwnerGrantHandler, ResourceOwnerGrant,\
-    Scope
+    Scope, RefreshToken, RefreshTokenHandler, ScopeGrant
 from oauth2.store import ClientStore, AuthCodeStore, AccessTokenStore
 from oauth2.error import OAuthInvalidError, OAuthUserError, OAuthClientError,\
-    ClientNotFoundError, UserNotAuthenticated
+    ClientNotFoundError, UserNotAuthenticated, AccessTokenNotFound
 from oauth2 import AuthorizationController, Client, AuthorizationCode,\
     AccessToken
+from oauth2.tokengenerator import TokenGenerator
 
-# Mock datetime.now()
-# see http://stackoverflow.com/questions/4481954/python-trying-to-mock-datetime-date-today-but-not-working
-import time
-def time():
+def mock_time():
     return 1000
-time.time = time
 
 class AuthorizationCodeGrantTestCase(unittest.TestCase):
     def test_create_auth_handler(self):
@@ -384,11 +381,13 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
         client_id = "abc"
         client_secret = "t%gH"
         code = "defg"
+        data = {"additional": "data"}
         redirect_uri = "http://callback"
         scopes = ["scope"]
         
         auth_code = Mock(AuthorizationCode)
         auth_code.code = code
+        auth_code.data = data
         auth_code.is_expired.return_value = False
         auth_code.redirect_uri = redirect_uri
         auth_code.scopes = scopes
@@ -420,6 +419,7 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
         self.assertEqual(handler.client_id, client_id)
         self.assertEqual(handler.client_secret, client_secret)
         self.assertEqual(handler.code, code)
+        self.assertEqual(handler.data, data)
         self.assertEqual(handler.redirect_uri, redirect_uri)
         self.assertEqual(handler.scopes, scopes)
         self.assertTrue(result)
@@ -680,17 +680,16 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
         self.assertEqual(error.error, "invalid_grant")
         self.assertEqual(error.explanation, "Authorization code has expired")
     
-    def test_process(self):
-        access_token = "abcd"
+    def test_process_no_refresh_token(self):
+        token_data = {"access_token": "abcd", "token_type": "Bearer"}
         client_id    = "efg"
+        data = {"additional": "data"}
         scopes       = ["scope"]
         
-        expected_body = {"access_token": access_token, "token_type": "Bearer"}
-        
         access_token_store_mock = Mock(spec=AccessTokenStore)
         
-        token_generator_mock = Mock(spec=["generate"])
-        token_generator_mock.generate.return_value = access_token
+        token_generator_mock = Mock(spec=TokenGenerator)
+        token_generator_mock.create_access_token_data.return_value = token_data
         
         response_mock = Mock(spec=Response)
         response_mock.body = None
@@ -701,33 +700,55 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
                                                 Mock(spec=ClientStore),
                                                 token_generator_mock)
         handler.client_id = client_id
-        handler.scopes    = scopes
+        handler.data = data
+        handler.scopes = scopes
         response = handler.process(Mock(spec=Request), response_mock, {})
         
         access_token, = access_token_store_mock.save_token.call_args[0]
         self.assertTrue(isinstance(access_token, AccessToken))
+        self.assertEqual(access_token.data, data)
+        self.assertEqual(access_token.grant_type, "authorization_code")
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.body, json.dumps(expected_body))
+        self.assertEqual(response.body, json.dumps(token_data))
         response_mock.add_header.assert_called_with("Content-type",
                                                     "application/json")
-    
-    def test_redirect_oauth_error(self):
-        error_identifier = "eid"
-        expected_content = {"error": error_identifier}
+    @patch("time.time", mock_time)
+    def test_process_with_refresh_token(self):
+        token_data = {"access_token": "abcd", "token_type": "Bearer",
+                      "refresh_token": "wxyz", "expires_in": 600}
+        client_id    = "efg"
+        data = {"additional": "data"}
+        scopes       = ["scope"]
         
-        error_mock = Mock(spec=OAuthUserError)
-        error_mock.error = error_identifier
+        access_token_store_mock = Mock(spec=AccessTokenStore)
+        
+        token_generator_mock = Mock(spec=TokenGenerator)
+        token_generator_mock.create_access_token_data.return_value = token_data
         
         response_mock = Mock(spec=Response)
+        response_mock.body = None
+        response_mock.status_code = None
         
-        handler = AuthorizationCodeTokenHandler(Mock(), Mock(), Mock(), Mock())
-        result = handler.redirect_oauth_error(error_mock, response_mock)
+        handler = AuthorizationCodeTokenHandler(access_token_store_mock,
+                                                Mock(spec=AuthCodeStore),
+                                                Mock(spec=ClientStore),
+                                                token_generator_mock)
+        handler.client_id = client_id
+        handler.data = data
+        handler.scopes = scopes
+        response = handler.process(Mock(spec=Request), response_mock, {})
         
+        access_token, = access_token_store_mock.save_token.call_args[0]
+        self.assertTrue(isinstance(access_token, AccessToken))
+        self.assertEqual(access_token.data, data)
+        self.assertEqual(access_token.grant_type, "authorization_code")
+        self.assertEqual(access_token.expires_at, 1600)
+        self.assertEqual(access_token.refresh_token,
+                         token_data["refresh_token"])
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.body, json.dumps(token_data))
         response_mock.add_header.assert_called_with("Content-type",
                                                     "application/json")
-        self.assertEqual(response_mock.status_code, 400)
-        self.assertEqual(response_mock.body, json.dumps(expected_content))
-        self.assertEqual(result, response_mock)
 
 class ImplicitGrantTestCase(unittest.TestCase):
     def test_create_matching_response_type(self):
@@ -806,6 +827,7 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         
         access_token, = access_token_store_mock.save_token.call_args[0]
         self.assertTrue(isinstance(access_token, AccessToken))
+        self.assertEqual(access_token.grant_type, "implicit")
         
         responseMock.add_header.assert_called_with("Location",
                                                    redirect_uri_with_token)
@@ -1028,8 +1050,9 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         access_token = "0aef"
         client_id    = "abcd"
         expected_response_body = {"access_token": access_token,
-                                  "token_type": "bearer"}
+                                  "token_type": "Bearer"}
         scopes       = ["scope"]
+        token_data = {"access_token": access_token, "token_type": "Bearer"}
         user         = {"id": 123}
         
         access_token_store_mock = Mock(AccessTokenStore)
@@ -1045,8 +1068,8 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         site_adapter_mock = Mock(SiteAdapter)
         site_adapter_mock.authenticate.return_value = user
         
-        token_generator_mock = Mock()
-        token_generator_mock.generate.return_value = access_token
+        token_generator_mock = Mock(spec=TokenGenerator)
+        token_generator_mock.create_access_token_data.return_value = token_data
         
         handler = ResourceOwnerGrantHandler(access_token_store_mock,
                                             Mock(ClientStore),
@@ -1058,9 +1081,60 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         
         site_adapter_mock.authenticate.assert_called_with(request_mock, {},
                                                           scopes)
-        token_generator_mock.generate.assert_called_with()
+        token_generator_mock.create_access_token_data.assert_called_with()
         access_token, = access_token_store_mock.save_token.call_args[0]
         self.assertTrue(isinstance(access_token, AccessToken))
+        self.assertEqual(access_token.grant_type,
+                         ResourceOwnerGrant.grant_type)
+        response_mock.add_header.assert_called_with("Content-Type",
+                                                    "application/json")
+        self.assertEqual(result.status_code, 200)
+        self.assertEqual(json.loads(result.body), expected_response_body)
+        self.assertEqual(result, response_mock)
+    
+    @patch("time.time", mock_time)
+    def test_process_with_refresh_token(self):
+        access_token = "0aef"
+        client_id    = "abcd"
+        expected_response_body = {"access_token": access_token,
+                                  "token_type": "Bearer",
+                                  "refresh_token": "wxyz", "expires_in": 600}
+        scopes       = ["scope"]
+        token_data = {"access_token": access_token, "token_type": "Bearer",
+                      "refresh_token": "wxyz", "expires_in": 600}
+        user         = {"id": 123}
+        
+        access_token_store_mock = Mock(AccessTokenStore)
+        
+        request_mock = Mock(Request)
+        
+        response_mock = Mock(Response)
+        
+        scope_handler_mock = Mock(Scope)
+        scope_handler_mock.scopes = scopes
+        scope_handler_mock.send_back = False
+        
+        site_adapter_mock = Mock(SiteAdapter)
+        site_adapter_mock.authenticate.return_value = user
+        
+        token_generator_mock = Mock(spec=TokenGenerator)
+        token_generator_mock.create_access_token_data.return_value = token_data
+        
+        handler = ResourceOwnerGrantHandler(access_token_store_mock,
+                                            Mock(ClientStore),
+                                            scope_handler_mock,
+                                            site_adapter_mock,
+                                            token_generator_mock)
+        handler.client_id = client_id
+        result = handler.process(request_mock, response_mock, {})
+        
+        site_adapter_mock.authenticate.assert_called_with(request_mock, {},
+                                                          scopes)
+        token_generator_mock.create_access_token_data.assert_called_with()
+        access_token, = access_token_store_mock.save_token.call_args[0]
+        self.assertTrue(isinstance(access_token, AccessToken))
+        self.assertEqual(access_token.refresh_token, token_data["refresh_token"])
+        self.assertEqual(access_token.expires_at, 1600)
         response_mock.add_header.assert_called_with("Content-Type",
                                                     "application/json")
         self.assertEqual(result.status_code, 200)
@@ -1072,8 +1146,9 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         client_id    = "abcd"
         scopes       = ["scope_read", "scope_write"]
         expected_response_body = {"access_token": access_token,
-                                  "token_type": "bearer",
+                                  "token_type": "Bearer",
                                   "scope": " ".join(scopes)}
+        token_data = {"access_token": access_token, "token_type": "Bearer"}
         
         response_mock = Mock(Response)
         
@@ -1081,8 +1156,8 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         scope_handler_mock.scopes = scopes
         scope_handler_mock.send_back = True
         
-        token_generator_mock = Mock()
-        token_generator_mock.generate.return_value = access_token
+        token_generator_mock = Mock(spec=TokenGenerator)
+        token_generator_mock.create_access_token_data.return_value = token_data
         
         handler = ResourceOwnerGrantHandler(Mock(AccessTokenStore),
                                             Mock(ClientStore),
@@ -1092,7 +1167,7 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         handler.client_id = client_id
         result = handler.process(Mock(Request), response_mock, {})
         
-        token_generator_mock.generate.assert_called_with()
+        token_generator_mock.create_access_token_data.assert_called_with()
         response_mock.add_header.assert_called_with("Content-Type",
                                                     "application/json")
         self.assertEqual(result.status_code, 200)
@@ -1294,6 +1369,372 @@ class ScopeTestCase(unittest.TestCase):
         e = expected.exception
         
         self.assertEqual(e.error, "invalid_scope")
+    
+    def test_compare_scopes_equal(self):
+        """
+        Scope.compare should use the same scopes if new and old scopes do not differ
+        """
+        scope = Scope(available=["a", "b"])
+        
+        scope.scopes = ["a", "b"]
+        
+        result = scope.compare(["a", "b"])
+        
+        self.assertTrue(result)
+        self.assertListEqual(scope.scopes, ["a", "b"])
+    
+    def test_compare_valid_scope_subset(self):
+        """
+        Scope.compare should set a new value for scopes attribute if the new scopes are a subset of the previously issued scopes
+        """
+        scope = Scope(available=["a", "b", "c"])
+        
+        scope.scopes = ["b", "c"]
+        
+        result = scope.compare(["a", "b", "c"])
+        
+        self.assertTrue(result)
+        self.assertListEqual(scope.scopes, ["b", "c"])
+    
+    def test_compare_invalid_scope_requested(self):
+        """
+        Scope.compare should thow an error if a scope is requested that is not contained in the previous scopes.
+        """
+        scope = Scope(available=["a", "b", "c"])
+        
+        scope.scopes = ["b", "c"]
+        
+        with self.assertRaises(OAuthInvalidError) as expected:
+            scope.compare(["a", "b"])
+        
+        e = expected.exception
+        
+        self.assertEqual(e.error, "invalid_scope")
+
+class RefreshTokenTestCase(unittest.TestCase):
+    def test_call(self):
+        """
+        RefreshToken should create a new instance of RefreshTokenHandler
+        """
+        path = "/token"
+        expires_in = 600
+        
+        access_token_store_mock = Mock()
+        client_store_mock = Mock()
+        scope_handler_mock = Mock()
+        token_generator_mock = Mock()
+        
+        controller_mock = Mock(spec=AuthorizationController)
+        controller_mock.token_path = path
+        controller_mock.access_token_store = access_token_store_mock
+        controller_mock.client_store = client_store_mock
+        controller_mock.scope_handler = scope_handler_mock
+        controller_mock.token_generator = token_generator_mock
+        controller_mock.tokens_expire_in = expires_in
+        
+        request_mock = Mock(spec=Request)
+        request_mock.path = path
+        request_mock.post_param.return_value = "refresh_token"
+        
+        grant = RefreshToken(expires_in=0)
+        
+        grant_handler = grant(request_mock, controller_mock)
+        
+        request_mock.post_param.assert_called_with("grant_type")
+        
+        self.assertTrue(isinstance(grant_handler, RefreshTokenHandler))
+        self.assertTrue(isinstance(grant_handler.scope_handler, Scope))
+        self.assertEqual(access_token_store_mock,
+                         grant_handler.access_token_store)
+        self.assertEqual(client_store_mock, grant_handler.client_store)
+        self.assertEqual(token_generator_mock, grant_handler.token_generator)
+    
+    def test_call_wrong_path(self):
+        """
+        RefreshToken should return 'None' if path in the request does not equal the token path
+        """
+        controller_mock = Mock(spec=AuthorizationController)
+        controller_mock.token_path = "/token"
+        
+        request_mock = Mock(spec=Request)
+        request_mock.path = "/authorize"
+        
+        grant = RefreshToken(expires_in=0)
+        
+        grant_handler = grant(request_mock, controller_mock)
+        
+        self.assertEqual(grant_handler, None)
+        
+    def test_call_other_grant_type(self):
+        """
+        RefreshToken should return 'None' if another grant type is requested
+        """
+        path = "/token"
+        
+        controller_mock = Mock(spec=AuthorizationController)
+        controller_mock.token_path = path
+        
+        request_mock = Mock(spec=Request)
+        request_mock.path = path
+        request_mock.get_param.return_value = "authorization_code"
+        
+        grant = RefreshToken(expires_in=0)
+        
+        grant_handler = grant(request_mock, controller_mock)
+        
+        self.assertEqual(grant_handler, None)
+
+class RefreshTokenHandlerTestCase(unittest.TestCase):
+    @patch("time.time", mock_time)
+    def test_process(self):
+        client_id = "testclient"
+        data = {"additional": "data"}
+        expires_in = 600
+        scopes = []
+        token = "abcdefg"
+        expected_response_body = {"access_token": token,
+                                  "expires_in": expires_in,
+                                  "token_type": "Bearer"}
+        expected_headers = {"Content-type": "application/json"}
+        
+        access_token_store_mock = Mock(spec=AccessTokenStore)
+        
+        response = Response()
+        
+        scope_handler_mock = Mock(spec=Scope)
+        scope_handler_mock.scopes = scopes
+        
+        token_generator_mock = Mock(spec=["generate"])
+        token_generator_mock.expires_in = expires_in
+        token_generator_mock.generate.return_value = token
+        
+        handler = RefreshTokenHandler(access_token_store=access_token_store_mock,
+                                      client_store=Mock(spec=ClientStore),
+                                      scope_handler=scope_handler_mock,
+                                      token_generator=token_generator_mock)
+        handler.client_id = client_id
+        handler.data = data
+        
+        result = handler.process(request=Mock(spec=Request),
+                                 response=response, environ={})
+        
+        access_token, = access_token_store_mock.save_token.call_args[0]
+        self.assertEqual(access_token.client_id, client_id)
+        self.assertEqual(access_token.grant_type, "refresh_token")
+        self.assertDictEqual(access_token.data, data)
+        self.assertEqual(access_token.token, token)
+        self.assertListEqual(access_token.scopes, scopes)
+        self.assertEqual(access_token.expires_at, 1600)
+        
+        self.assertEqual(result, response)
+        self.assertDictContainsSubset(expected_headers, result.headers)
+        self.assertEqual(json.dumps(expected_response_body), result.body)
+    
+    @patch("time.time", mock_time)
+    def test_read_validate_params(self):
+        client_id = "client"
+        client_secret = "secret"
+        data = {"additional": "data"}
+        original_token = "sd3f3j"
+        refresh_token = "s74jf"
+        scopes=[]
+        
+        access_token = AccessToken(client_id=client_id, token=original_token,
+                                   grant_type=RefreshToken.grant_type,
+                                   data=data, expires_at=1234, scopes=scopes)
+        
+        access_token_store_mock = Mock(AccessTokenStore)
+        access_token_store_mock.fetch_by_refresh_token.return_value = access_token
+        
+        client = Client(identifier=client_id, secret=client_secret,
+                        redirect_uris=[])
+        
+        client_store_mock = Mock(spec=ClientStore)
+        client_store_mock.fetch_by_client_id.return_value = client
+        
+        request_mock = Mock(spec=Request)
+        request_mock.post_param.side_effect = [client_id, client_secret,
+                                               refresh_token]
+        
+        scope_handler_mock = Mock(spec=Scope)
+        
+        handler = RefreshTokenHandler(access_token_store=access_token_store_mock,
+                                      client_store=client_store_mock,
+                                      scope_handler=scope_handler_mock,
+                                      token_generator=Mock())
+        
+        handler.read_validate_params(request=request_mock)
+        
+        request_mock.post_param.assert_has_calls([call("client_id"),
+                                                  call("client_secret"),
+                                                  call("refresh_token")])
+        access_token_store_mock.fetch_by_refresh_token.assert_called_with(refresh_token)
+        client_store_mock.fetch_by_client_id.assert_called_with(client_id)
+        scope_handler_mock.parse.assert_called_with(request_mock)
+        scope_handler_mock.compare.assert_called_with(scopes)
+        
+        self.assertEqual(handler.client_id, client_id)
+        self.assertEqual(handler.data, data)
+        self.assertEqual(handler.refresh_token, refresh_token)
+    
+    
+    def test_read_validate_params_no_client_id(self):
+        request_mock = Mock(spec=Request)
+        request_mock.post_param.return_value = None
+        
+        handler = RefreshTokenHandler(access_token_store=Mock(),
+                                      client_store=Mock(),
+                                      scope_handler=Mock(),
+                                      token_generator=Mock())
+        
+        with self.assertRaises(OAuthInvalidError) as expected:
+            handler.read_validate_params(request_mock)
+        
+        e = expected.exception
+        
+        self.assertEqual(e.error, "invalid_request")
+        self.assertEqual(e.explanation, "Missing client_id in request body")
+    
+    def test_read_validate_params_no_client_secret(self):
+        request_mock = Mock(spec=Request)
+        request_mock.post_param.side_effect = ["abc", None]
+        
+        handler = RefreshTokenHandler(access_token_store=Mock(),
+                                      client_store=Mock(),
+                                      scope_handler=Mock(),
+                                      token_generator=Mock())
+        
+        with self.assertRaises(OAuthInvalidError) as expected:
+            handler.read_validate_params(request_mock)
+        
+        e = expected.exception
+        
+        self.assertEqual(e.error, "invalid_request")
+        self.assertEqual(e.explanation, "Missing client_secret in request body")
+    
+    def test_read_validate_params_no_refresh_token(self):
+        request_mock = Mock(spec=Request)
+        request_mock.post_param.side_effect = ["abc", "xyz", None]
+        
+        handler = RefreshTokenHandler(access_token_store=Mock(),
+                                      client_store=Mock(),
+                                      scope_handler=Mock(),
+                                      token_generator=Mock())
+        
+        with self.assertRaises(OAuthInvalidError) as expected:
+            handler.read_validate_params(request_mock)
+        
+        e = expected.exception
+        
+        self.assertEqual(e.error, "invalid_request")
+        self.assertEqual(e.explanation, "Missing refresh_token in request body")
+    
+    def test_read_validate_params_client_not_found(self):
+        request_mock = Mock(spec=Request)
+        request_mock.post_param.side_effect = ["abc", "xyz", "uuu"]
+        
+        client_store_mock = Mock(spec=ClientStore)
+        client_store_mock.fetch_by_client_id.side_effect = ClientNotFoundError
+        
+        handler = RefreshTokenHandler(access_token_store=Mock(),
+                                      client_store=client_store_mock,
+                                      scope_handler=Mock(),
+                                      token_generator=Mock())
+        
+        with self.assertRaises(OAuthInvalidError) as expected:
+            handler.read_validate_params(request_mock)
+        
+        e = expected.exception
+        
+        self.assertEqual(e.error, "invalid_request")
+        self.assertEqual(e.explanation, "Unknown client")
+    
+    def test_read_validate_params_invalid_client_secret(self):
+        client_id = "abc"
+        secret_expected = "mno"
+        secret_actual = "xyz"
+        
+        request_mock = Mock(spec=Request)
+        request_mock.post_param.side_effect = [client_id, secret_actual, "uuu"]
+        
+        client = Client(identifier=client_id, secret=secret_expected,
+                        redirect_uris=[])
+        
+        client_store_mock = Mock(spec=ClientStore)
+        client_store_mock.fetch_by_client_id.return_value = client
+        
+        handler = RefreshTokenHandler(access_token_store=Mock(),
+                                      client_store=client_store_mock,
+                                      scope_handler=Mock(),
+                                      token_generator=Mock())
+        
+        with self.assertRaises(OAuthInvalidError) as expected:
+            handler.read_validate_params(request_mock)
+        
+        e = expected.exception
+        
+        self.assertEqual(e.error, "invalid_request")
+        self.assertEqual(e.explanation, "Invalid client secret")
+    
+    def test_read_validate_params_invalid_refresh_token(self):
+        client_id = "abc"
+        secret = "xyz"
+        
+        access_token_store_mock = Mock(spec=AccessTokenStore)
+        access_token_store_mock.fetch_by_refresh_token.side_effect = AccessTokenNotFound
+        
+        request_mock = Mock(spec=Request)
+        request_mock.post_param.side_effect = [client_id, secret, "uuu"]
+        
+        client = Client(identifier=client_id, secret=secret, redirect_uris=[])
+        
+        client_store_mock = Mock(spec=ClientStore)
+        client_store_mock.fetch_by_client_id.return_value = client
+        
+        handler = RefreshTokenHandler(access_token_store=access_token_store_mock,
+                                      client_store=client_store_mock,
+                                      scope_handler=Mock(),
+                                      token_generator=Mock())
+        
+        with self.assertRaises(OAuthInvalidError) as expected:
+            handler.read_validate_params(request_mock)
+        
+        e = expected.exception
+        
+        self.assertEqual(e.error, "invalid_request")
+        self.assertEqual(e.explanation, "Invalid refresh token")
+    
+    @patch("time.time", mock_time)
+    def test_read_validate_params_expired_access_token(self):
+        client_id = "abc"
+        secret = "xyz"
+        
+        access_token_mock = Mock(spec=AccessToken)
+        access_token_mock.expires_at = 900
+        
+        access_token_store_mock = Mock(spec=AccessTokenStore)
+        access_token_store_mock.fetch_by_refresh_token.return_value = access_token_mock
+        
+        request_mock = Mock(spec=Request)
+        request_mock.post_param.side_effect = [client_id, secret, "uuu"]
+        
+        client = Client(identifier=client_id, secret=secret, redirect_uris=[])
+        
+        client_store_mock = Mock(spec=ClientStore)
+        client_store_mock.fetch_by_client_id.return_value = client
+        
+        handler = RefreshTokenHandler(access_token_store=access_token_store_mock,
+                                      client_store=client_store_mock,
+                                      scope_handler=Mock(),
+                                      token_generator=Mock())
+        
+        with self.assertRaises(OAuthInvalidError) as expected:
+            handler.read_validate_params(request_mock)
+        
+        e = expected.exception
+        
+        self.assertEqual(e.error, "invalid_request")
+        self.assertEqual(e.explanation, "Invalid refresh token")
 
 if __name__ == "__main__":
     unittest.main()
